# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_utils.ipynb.

# %% auto 0
__all__ = ['bar_plot']

# %% ../nbs/00_utils.ipynb 3
import pkgutil
import pandas as pd
from typing import Optional, List

def rawdata(
        all: bool = True, # If True, returns all dataframes. If False, returns only the employee_details dataframe
        tables: Optional[List] = None # If not None, returns only the dataframes in the list
        ) -> dict: # Returns a dictionary of dataframes
    """
    Returns a dictionary of dataframes containing the data for the project
    
    Internal export - pkgutil component can only run in the context of a module
    """
    if all and tables is None:
        return {
            'employee_details': pd.read_excel(pkgutil.get_data(__name__, '_data/Employee_details.xlsx')),
            'glossary': pd.read_excel(pkgutil.get_data(__name__, '_data/Glossary.xlsx')),
            'manager_details': pd.read_excel(pkgutil.get_data(__name__, '_data/Manager_details.xlsx')),
            'manager_mapping': pd.read_excel(pkgutil.get_data(__name__, '_data/Manager_mapping.xlsx')),
            'promo_meta_data': pd.read_excel(pkgutil.get_data(__name__, '_data/promo_meta_data.xlsx'))
        }
    elif tables is not None:
        return {table: pd.read_excel(pkgutil.get_data(__name__, f'_data/{table}.xlsx')) for table in tables}
    else:
        raise ValueError('Either all or tables must be specified')

# %% ../nbs/00_utils.ipynb 4
import matplotlib.ticker as mtick
import matplotlib.pyplot as plt
import numpy as np
import seaborn as sns

def bar_plot(
        x_values: np.ndarray, # The x-axis values
        y_values: np.ndarray, # The y-axis values
        type: str = 'Job Family' # The category/type of the x-axis values
    ) -> None:
    "Helper function to plot a bar plot with the attrition rate"
    plt.figure(figsize=(7, 3))
    custom_params = {"axes.spines.right": False, "axes.spines.top": False}
    sns.set_theme(style="ticks", rc=custom_params)

    normalized_values = (y_values - np.min(y_values)) / (np.max(y_values) - np.min(y_values))
    ax = sns.barplot(x=x_values, y=y_values, palette=sns.color_palette("coolwarm", as_cmap=True)(normalized_values))

    # Add the values on top of the bars
    for p in ax.patches:
        height = p.get_height()
        ax.text(p.get_x()+p.get_width()/2.,
                height + 0.01,
                '{:1.1f}%'.format(height*100),
                ha="center", fontsize=11)
    
    plt.title(f'Attrition Rate by {type}')
    plt.xlabel('')
    plt.ylabel('Attrition Rate')
    plt.xticks(rotation=45, ha='right')

    # Change y-axis to percentage format
    ax.yaxis.set_major_formatter(mtick.PercentFormatter(1.0))


    plt.show()    



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_data.ipynb.

# %% auto 0
__all__ = ['AData']

# %% ../nbs/01_data.ipynb 3
from .utils import rawdata
import pandas as pd
from typing import Optional, List, Callable

class AData:
    def __init__(self, 
                 all: bool = True, # If True, all data is loaded. If False, only a subset is loaded
                 tables: Optional[List] = None # If not None, returns only the dataframes in the list
                 ) -> None:
        "Does stuff"
        self.all = all
        self.tables = tables
        
        self.raw_data = rawdata(all=all, tables=tables)
        self.cleaned_data = {}

        # Map of tables to functions to clean them
        # - order of list is order the cleaning functions is applied
        self.table_func_map = {
            'employee_details': [
                {'func':self.remove_missing_by_col,
                 'kwargs':{'colname': 'emp_id'}
                },
                {'func':self.remove_duplicate_by_col,
                 'kwargs':{'colname': 'emp_id'}
                }
            ],
            'manager_details': [
                {'func':self.remove_missing_by_col,
                 'kwargs':{'colname': 'mgr_id'}
                },
                {'func':lambda x: x.groupby('mgr_id')['exp_as_mgr'].mean().reset_index(),
                 'kwargs':{}
                }
            ],
            'manager_mapping': [
                {'func':self.remove_missing_by_col,
                 'kwargs':{'colname': 'mgr_id'}
                },
                {'func':self.remove_duplicate_by_col,
                 'kwargs':{'colname': 'emp_id'}
                },
            ]
        }

        self.clean_data()
    
    def __repr__(self) -> str:
        return f"AData(all={self.all}, tables={self.tables})"
    
    def __str__(self) -> str:
        return f"AData(all={self.all}, tables={self.tables})"

    def _clean_data(self, 
                    func: Callable, # Function to clean data, should 
                    tables: List[str], # List of tables to clean
                    **kwargs # Additional arguments to pass to the function
                    ) -> None:
        "Cleans data using the function passed"
        for table in tables:
            if table not in self.raw_data.keys():
                raise ValueError(f"Table {table} not found in raw data")
            
            if table not in self.cleaned_data.keys():
                self.cleaned_data[table] = func(self.raw_data[table], **kwargs)
            else:
                self.cleaned_data[table] = func(self.cleaned_data[table], **kwargs)

    def clean_data(self) -> None:
        "Cleans data"
        
        for table_name in self.raw_data.keys():
            # if no cleaning functions are specified for the table, add the raw data to cleaned data
            if table_name not in self.table_func_map.keys():
                self.cleaned_data[table_name] = self.raw_data[table_name]
                continue

            for func in self.table_func_map[table_name]:
                self._clean_data(func=func['func'], tables=[table_name], **func['kwargs'])
    
    @staticmethod
    def remove_missing_by_col(
        data: pd.DataFrame, # Dataframe to remove records from 
        colname: str, # Column name to remove records by
        ):
        "Removes records with missing values in the specified column"
        return data[~data.loc[:, colname].isna()]

    @staticmethod
    def remove_duplicate_by_col(
        data: pd.DataFrame, # Dataframe to remove records from 
        colname: str, # Column name to remove records by
        ):
        "Removes duplicate records in the specified column"
        return data.drop_duplicates(subset=colname)





# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/atlassianhw',
                'doc_host': 'https://jazon33y.github.io',
                'git_url': 'https://github.com/jazon33y/atlassianhw',
                'lib_path': 'atlassianhw'},
  'syms': { 'atlassianhw.data': { 'atlassianhw.data.AData': ('data.html#adata', 'atlassianhw/data.py'),
                                  'atlassianhw.data.AData.__init__': ('data.html#adata.__init__', 'atlassianhw/data.py'),
                                  'atlassianhw.data.AData.__repr__': ('data.html#adata.__repr__', 'atlassianhw/data.py'),
                                  'atlassianhw.data.AData.__str__': ('data.html#adata.__str__', 'atlassianhw/data.py'),
                                  'atlassianhw.data.AData._clean_data': ('data.html#adata._clean_data', 'atlassianhw/data.py'),
                                  'atlassianhw.data.AData.clean_data': ('data.html#adata.clean_data', 'atlassianhw/data.py'),
                                  'atlassianhw.data.AData.remove_duplicate_by_col': ( 'data.html#adata.remove_duplicate_by_col',
                                                                                      'atlassianhw/data.py'),
                                  'atlassianhw.data.AData.remove_missing_by_col': ( 'data.html#adata.remove_missing_by_col',
                                                                                    'atlassianhw/data.py')},
            'atlassianhw.utils': { 'atlassianhw.utils.bar_plot': ('utils.html#bar_plot', 'atlassianhw/utils.py'),
                                   'atlassianhw.utils.rawdata': ('utils.html#rawdata', 'atlassianhw/utils.py')}}}
